#include "stdafx.h"

//-----------------------------------------------------------------------------
void Swap( double * d1, double * d2 ) { 
	double d = *d1;
	*d1 = *d2;
	*d2 = d;
	}
	
void Swap( int * i1, int * i2 ) { 
	int i = *i1;
	*i1 = *i2;
	*i2 = i;
	}

void Swap( int iItems, double * ad0, double * ad1 ) { 
	double d;
	for( int i=0; i<iItems; ++i ) {
		d=ad0[i];
		ad0[i]=ad1[i];
		ad1[i]=d;
		}
	}

void Swap( int iItems, int * ai0, int * ai1 ) { 
	int ii;
	for( int i=0; i<iItems; ++i ) {
		ii=ai0[i];
		ai0[i]=ai1[i];
		ai1[i]=ii;
		}
	}

//-----------------------------------------------------------------------------
int DistanceFromPointToLine( int iXpt, int iYpt, int iX1, int iY1, int iX2, int iY2 ) { 
	int idx=iX2-iX1;
	int idy=iY2-iY1;
	return MulDiv( abs(idx*(iY1-iYpt)-idy*(iX1-iXpt)), 1, int( sqrt(double(idx*idx+idy*idy)) ) ); ;
	}
	
//-----------------------------------------------------------------------------
double log2( double d ) { 
	return log10(d)/log10(2.0);
	}

//----- calculate ln( n! ) as sum[i=2..n]ln(i) to avoid overflow.
/*
double LogFactorial( int iVal ){
	double dResult = 0;
	for( int i=iVal; i>1; --i )
		dResult += log(double(i));
	return dResult;
	}
*/	
double LogFactorial( int iVal ){
	//- Favor speed.  First 15 are precalc'd (generally not calculating for high values).
	// 128 bytes storage is trivial.
	const double adPrecalc[15]={
			0.000000000000000,  0.000000000000000,   0.693147180559945,  1.791759469228050, 
			3.178053830347950,  4.787491742782050,   6.579251212010100,  8.525161361065410,
			10.604602902745300, 12.801827480081500, 15.104412573075500, 17.502307845873900,
			19.987214495661900, 22.552163853123400, 25.191221182738700 
			};
	
	if( iVal<15 )
		return adPrecalc[max(0,iVal)];

	double dResult=adPrecalc[14];
	for( int i=iVal; i>14; --i )
		dResult += log(double(i));
	return dResult;
	}

//----- calculate i! / ( j! * (i-j)! )
double BinCoeff( int i, int j ) { 
	return exp( LogFactorial(i) - LogFactorial(i-j) - LogFactorial(j) );
	}
	
//----- same as BinCoeff, may overflow for i>=34 	
int BinCoeffInt( int i, int j ) { 
	//-- for large values, result can overflow integer math ex: (34|17) = 2333606220 = 1.08*2^31
	int iResult=1;
	int iDen=2;			// denominator factors from 1 to j
	j=j>i-j?i-j:j;		// do the lesser calculation of (i|j) and (i|i-j) since they are the same
	for( int iNum=i-j+1; iNum<=i; ++iNum ) {	// numerator factors from i-j to i 
		iResult*=iNum;
		while( iResult%iDen==0 && iDen<=j )	// avoid overflow by early division when accurate
			iResult/=iDen++;		// eventually all will be caught - result is always integer
		}
	return iResult;
	}	

//-----------------------------------------------------------------------
// Returns max element in x[0..n-1] and sets imaxpos to position of its 
// first occurrence.
int FindMaxPos( int iData, double * adData, bool bMax, double * pdMax ) { 
	int iPos=0;
	double d=adData[iPos];
	for( int i=1; i<iData; ++i ) 
		if( (adData[i]>d) == bMax ) 
			d=adData[iPos=i];
	if( pdMax != NULL )
		*pdMax=adData[iPos];
	return iPos;
	}

//-----------------------------------------------------------------------
// Find Range (minimum, maximum) of a dataset
void FindRange( int iData, double * pdData, double * pdMin, double * pdMax, int * piMinPos, int * piMaxPos ) { 
	int iMinPos=0, iMaxPos=0;
	double dMin,dMax;
	dMin=dMax=*pdData;
	for( int i=1; i<iData; ++i ) {
		++pdData;
		if( *pdData<dMin ) {
			dMin=*pdData;
			iMinPos=i;
			}
		else if( *pdData>dMax ) {
			dMax=*pdData;
			iMaxPos=i;
			}
		}
	*pdMin=dMin;
	*pdMax=dMax;
	if( piMinPos!=NULL ) 
		*piMinPos=iMinPos;
	if( piMaxPos!=NULL ) 
		*piMaxPos=iMaxPos;
	}

//-----------------------------------------------------------------------
double GaussianWeightFactor( double dMean, double dStdDev, double dVal ){
	const double EPS = 0.00001;
	const double SQRT2PI = sqrt( M_PI+M_PI );
	
	//-- for SD=0, within EPS of mean should be something like 1/SQRT2PI/EPS
	if ( dStdDev == 0.0 )
		return fabs(dVal-dMean) < EPS ? 100.0 : 0.0 ;
		
	
	double dStds = (dVal-dMean)/dStdDev;		// Number of standard deviations away from mean
	return max( exp(dStds*dStds/-2.0) / (SQRT2PI*dStdDev), 1.0e-8 );
	}

//-----------------------------------------------------------------------
double Pythag( int iLen, double * ad ) { 
	double dSum=0.0;
	for( int i=0; i<iLen; ++i ) 
		dSum += ad[i]*ad[i];
	return dSum>0.0 ? sqrt( dSum ) : 0.0;
	}

//-----------------------------------------------------------------------
void Scale( int iLen, double * ad, double dMult ) { 
	for( int i=0; i<iLen; ++i ) 
		ad[i]*=dMult;
	}
	
//-----------------------------------------------------------------------
int lnsrch( 
		int n, 
		double * xold,					// starting pt.
		double fold,					// log likelihood at starting pt
		double * g,						// gradient at starting pt
		double * p,						// direction
		double * x,						// new pt.
		double * f,						// sufficiently changed LL at new pt.
		double dStepMax,				// limit step size 
		EvalFunc func, 
		void * pCargo, 
		bool * pbStopped ) { 		

	const double ALF  = 1.0e-4;
	const double TOLX = 1.0e-7;
	const double VERY_SMALL = 1.0e-15;

	int i; 
	double a, alam, alam2, alamin, b, disc, f2, fold2, rhs1, rhs2, tmplam;
	
	//-- limit step distance to dStepmax.  
	double dSum = Pythag( n, p );		// dSum = sqrt( sum( p[0..n] ^ 2 ) ) 
	if( dSum > dStepMax )
		Scale( n, p, dStepMax/dSum );	// p[0..n] *= dStepMax/dSum 
	
	//-- 		
	double dSlope = SumProduct( n, g, p );

	double dTest=VERY_SMALL; 
	for( i=0; i<n; i++ ) 
		dTest=max( dTest, fabs(p[i])/max(fabs(xold[i]),1.0)) ;
	alamin=TOLX/dTest;
	alam=1.0;

	for( ; ; ) {
		if( pbStopped && *pbStopped ) {			// check if user cancelled
			return LNSRCH_CANC;
			}
			
		vec_axplusb( n, alam, p, xold, x );		// x[i] = alam * p[i] + xold[i]	: step alam(0.0:1.0) of full step.

		*f = func( x, pCargo );

		if( alam < alamin ) {
			veccopy( n, xold, x );
			return LNSRCH_CHCK;
			}
			
		if( *f <= fold+ALF*alam*dSlope ) {
			return LNSRCH_EXIT;
			}
			
		//-- backtrack
		if( alam == 1.0 )						// first time
			tmplam = -dSlope/(2.0*(*f-fold-dSlope));
		else {									// subsequent
			rhs1 = *f - fold - alam * dSlope ;
			rhs2 = f2 - fold2 - alam2 * dSlope ;
			a = (rhs1/(alam*alam)-rhs2/(alam2*alam2)) / (alam-alam2) ;
			b = (-alam2*rhs1/(alam*alam)+alam*rhs2/(alam2*alam2)) / (alam-alam2) ;
			if (a == 0.0)
				tmplam = -dSlope/(2.0*b);
			else {
				disc=b*b-3.0*a*dSlope;
				if( disc<0.0 ) {
					return LNSRCH_CHCK;
					}
				tmplam=(-b+sqrt(disc))/(3.0*a);
				}
			tmplam = min( tmplam, 0.5*alam );
			}
			
		alam2=alam;
		f2 = *f;
		fold2=fold;
		alam=max(tmplam,0.1*alam);
		}
		
	return LNSRCH_EXIT;
	}

int dfpmin( double * p,					// starting pt. (Z) / result
			int n,						// size of p (iZSize)
			double gtol,				// convergence requirement for gradient
			int iMaxIter,				// max iterations
			double * pdResult,			// minimum value achieved
			EvalFunc func,				// function to minimize
			EvalGrad dfunc,				// gradient calculation				
			void * pCargo,				// data to pass to lnsrch (CMil *)
			int * piIter, 				// iterations
			bool * pbStopped ) {		// user cancel flag 
				
	const int NMAX = 50;
	//const int ITMAX = 200;
	const double STPMAX = 100.0;
	const double EPS = 3.0e-8;
	const double TOLX = 4 * EPS ;

	TNT::Array1D<double> g( n, 0.0 );
	TNT::Array1D<double> dg( n, 0.0 );
	TNT::Array1D<double> hdg( n, 0.0 );
	TNT::Array1D<double> pnew( n, 0.0 );
	TNT::Array1D<double> xi( n, 0.0 );
	TNT::Array2D<double> hessian( n, n, 0.0 );	

	int i, j; 
	double fac, fad, fae, sumdg, sumxi, sum=0.0, test, den;
	double fp = func( p , pCargo );
	dfunc( p, g, pCargo  );

	double dSum=0.0;
	for( i=0; i<n; ++i ) {
		hessian[i][i] = 1.0;
		xi[i] = -g[i];
		dSum += p[i]*p[i];
		}
	double stpmax = STPMAX * max(sqrt(dSum),(double)n);

	//-- iterate
	for( int iIter=1; iIter<=iMaxIter; iIter++ ) {
		*piIter=iIter;
		int iResult = lnsrch( n, p, fp, g, xi, pnew, pdResult, stpmax, func, pCargo, pbStopped );

		if( pbStopped && *pbStopped ) 
			return DFPMIN_CANC;

		fp=*pdResult;		// save lnsrch func eval for next search

		vec_axplusb( n, -1.0, p, (double *) pnew, (double *)xi );	// xi[i=0..n-1] = pnew[i]-p[i]
		veccopy( n, (double *)pnew, p );						// p[i=0..n-1] = pnew[i]

		//-- test for convergence on delta X
		test=0.0;
		for( i=0; i<n; ++i ) 
			test = max( test, fabs(xi[i])/max(abs(p[i]),1.0) );
		if( test < TOLX ) 
			return DFPMIN_EXIT;

		//-- save old gradient , get new gradient
		veccopy( n, (double *)g, (double *)dg );
		dfunc( p, g, pCargo);

		//-- Test for convergence on zero gradient 
		test=0.0;
		den = max( fp, 1.0 );
		for( i=0; i<n; ++i ) 
			test = max( test, abs(g[i])*max(abs(p[i]),1.0)/den );
		if( test < gtol ) 
			return DFPMIN_GTOL;
		
		//-- compute difference of gradients and difference times current matrix
		for( i=0; i<n; ++i ) 
			dg[i]=g[i]-dg[i];
		matmult( n, n, (double **) hessian, (double *)dg, (double *)hdg );

		//-- calculate dot products for the denominators
		fac=fae=sumdg=sumxi=0.0;
		for( i=0; i<n; i++ ) {
			fac += dg[i]*xi[i];
			fae += dg[i]*hdg[i];
			sumdg += ( dg[i] * dg[i] );
			sumxi += ( xi[i] * xi[i] );
			}

		//-- skip update if fac not sufficiently positive
		if( fac*fac > EPS*sumdg*sumxi ) {
			fac=1.0/fac;
			fad=1.0/fae;
			for (i=0; i<n; i++)
				dg[i]=fac*xi[i]-fad*hdg[i];
			for( i=0; i<n; i++ ) 
				for( j=0; j<n; j++ ) 
					hessian[i][j] += fac*xi[i]*xi[j] - fad*hdg[i]*hdg[j] + fae*dg[i]*dg[j];
			}

		//-- Calc next direction to go : xi[i=0..n-1] = - ( hessian * g )
		for( i=0; i<n; i++ ) {
			dSum=0.0;
			for( j=0; j<n; j++ )
				dSum += hessian[i][j]*g[j];
			xi[i]=-dSum;
			}
		}

	return DFPMIN_ITER;
	}

